package json

import (
	"fmt"
	"strconv"
	"strings"
	"time"
)

var (
	MYSQL_TYPE_DECIMAL       = 0
	MYSQL_TYPE_INT8          = 1
	MYSQL_TYPE_INT16         = 2
	MYSQL_TYPE_INT32         = 3
	MYSQL_TYPE_FLOAT         = 4
	MYSQL_TYPE_DOUBLE        = 5
	MYSQL_TYPE_NULL          = 6
	MYSQL_TYPE_TIMESTAMP     = 7
	MYSQL_TYPE_INT64         = 8
	MYSQL_TYPE_INT24         = 9
	MYSQL_TYPE_DATE          = 10
	MYSQL_TYPE_TIME          = 11
	MYSQL_TYPE_DATETIME      = 12
	MYSQL_TYPE_YEAR          = 13
	MYSQL_TYPE_DATE_NEW      = 14
	MYSQL_TYPE_VARCHAR       = 15
	MYSQL_TYPE_BIT           = 16
	MYSQL_TYPE_TIMESTAMP_NEW = 17
	MYSQL_TYPE_DATETIME_NEW  = 18
	MYSQL_TYPE_TIME_NEW      = 19
	MYSQL_TYPE_JSON          = 245
	MYSQL_TYPE_DECIMAL_NEW   = 246
	MYSQL_TYPE_ENUM          = 247
	MYSQL_TYPE_SET           = 248
	MYSQL_TYPE_TINY_BLOB     = 249
	MYSQL_TYPE_MEDIUM_BLOB   = 250
	MYSQL_TYPE_LONG_BLOB     = 251
	MYSQL_TYPE_BLOB          = 252
	MYSQL_TYPE_VAR_STRING    = 253
	MYSQL_TYPE_STRING        = 254
	MYSQL_TYPE_GEOMETRY      = 255
)

// 当mysql源库参数binlog_row_image设置为minimal时，
// binlog中update记录的未修改字段的值是缺失的，
// 这时用这个特定字符串代表缺失字段的值。
const JSON_NA_VALUE = "##DTS_NA_VALUE##"

// Decode data 可能有两种类型：string、或nil
func Decode(data interface{}, field *Field) string {
	if data == nil {
		return "NULL"
	}
	d := data.(string)

	if d == JSON_NA_VALUE {
		return "DEFAULT"
	}

	// 优先处理二进制类型。因为他们被处理成了hex string
	if strings.Contains(field.OriginalType, "binary") ||
		strings.Contains(field.OriginalType, "blob") ||
		int(field.DataTypeNumber) == MYSQL_TYPE_GEOMETRY {
		return "x'" + d + "'"
	}

	switch int(field.DataTypeNumber) {
	case MYSQL_TYPE_DECIMAL, MYSQL_TYPE_DECIMAL_NEW,
		MYSQL_TYPE_INT8, MYSQL_TYPE_INT16, MYSQL_TYPE_INT24,
		MYSQL_TYPE_INT32, MYSQL_TYPE_INT64, MYSQL_TYPE_FLOAT,
		MYSQL_TYPE_DOUBLE, MYSQL_TYPE_BIT, MYSQL_TYPE_NULL:
		return d
	case MYSQL_TYPE_JSON:
		return "'" + d + "'"
	case MYSQL_TYPE_TIMESTAMP, MYSQL_TYPE_TIMESTAMP_NEW: // 时间戳返回的是毫秒数
		timestamp, err := strconv.ParseInt(d, 10, 64)
		if err != nil {
			panic(fmt.Sprintf("error to parse int64: %s, error: %v", data, err))
		}
		t := time.Unix(timestamp/1000, timestamp%1000*1000000)
		return "_binary'" + t.Format("2006-01-02 15:04:05.000") + "'"
	case MYSQL_TYPE_DATETIME, MYSQL_TYPE_DATETIME_NEW:
		tstr := strings.Split(d, " ")
		if len(tstr) == 3 { // datetime 可能带时区信息
			return "_binary'" + tstr[0] + " " + tstr[1] + "'"
		}
		return "_binary'" + d + "'"
	default:
		return "_binary'" + d + "'"
	}

}
